package com.chelvc.framework.base.config;

import java.util.Collection;

import com.chelvc.framework.common.util.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import ma.glasnost.orika.MapperFactory;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.CollectionUtils;

/**
 * 对象复制处理器配置
 *
 * @author Woody
 * @date 2021/7/12
 */
@Slf4j
@Configuration
public class CopyingConfigurer implements ApplicationContextAware {
    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        Collection<CopierFactoryConfigurer> configurers =
                applicationContext.getBeansOfType(CopierFactoryConfigurer.class).values();
        if (!CollectionUtils.isEmpty(configurers)) {
            log.info("Initialize object copier configurers: {}", configurers);
            MapperFactory factory = ObjectUtils.getCopierBuilder();
            configurers.forEach(configurer -> configurer.configure(factory));
        }
    }

    /**
     * 对象拷贝工厂配置接口
     */
    public interface CopierFactoryConfigurer {
        /**
         * 对象映射工厂配置
         *
         * @param factory 对象映射工厂实例
         */
        void configure(MapperFactory factory);
    }
}
