package com.chelvc.framework.base.interceptor;

import ch.qos.logback.classic.Level;
import ch.qos.logback.classic.Logger;
import ch.qos.logback.classic.turbo.TurboFilter;
import ch.qos.logback.core.spi.FilterReply;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.StringUtils;
import org.slf4j.Marker;

/**
 * 动态日志处理拦截器
 *
 * @author Woody
 * @date 2023/9/1
 */
public class DynamicLoggingInterceptor extends TurboFilter {
    @Override
    public FilterReply decide(Marker marker, Logger logger, Level level, String format, Object[] params, Throwable t) {
        if (logger == null || level == null) {
            return FilterReply.NEUTRAL;
        }

        // 排除Spring的相关类的日志级别判断，防止出现递归
        String path = logger.getName();
        if (StringUtils.isEmpty(path) || path.startsWith("org.springframework.")) {
            return FilterReply.NEUTRAL;
        }

        // 动态判断日志级别
        Level preset = ApplicationContextHolder.getSafeProperty("logging.level." + path, Level.class);
        if (preset == null) {
            return FilterReply.NEUTRAL;
        } else if (level.isGreaterOrEqual(preset)) {
            return FilterReply.ACCEPT;
        }
        return FilterReply.DENY;
    }
}
