package com.chelvc.framework.feign.interceptor;

import java.util.Enumeration;
import java.util.Map;
import java.util.Objects;
import javax.servlet.http.HttpServletRequest;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.model.Session;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.feign.util.FeignUtils;
import com.google.common.collect.Maps;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * Feign请求头处理拦截器
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
@Component
public class FeignHeaderInterceptor implements RequestInterceptor {
    @Override
    public void apply(RequestTemplate template) {
        // 加载当前请求头
        Map<String, Object> headers = Maps.newHashMap();
        HttpServletRequest request = SessionContextHolder.getRequest();
        Enumeration<String> names = ObjectUtils.ifNull(request, HttpServletRequest::getHeaderNames);
        if (names != null) {
            while (names.hasMoreElements()) {
                String key = names.nextElement();
                headers.put(key, request.getHeader(key));
            }
        }

        // 重置会话认证请求头
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            headers.put(SessionContextHolder.HEADER_ID, session.getId());
            headers.put(SessionContextHolder.HEADER_MOBILE, session.getMobile());
            headers.put(SessionContextHolder.HEADER_BUSINESS, session.getBusiness());
        }

        // 初始化Feign请求头
        boolean debug = log.isDebugEnabled();
        headers.forEach((key, value) -> {
            if (Objects.nonNull(value)) {
                if (debug) {
                    log.debug("Initializing feign request header: {} = {}", key, value);
                }
                template.header(key, String.valueOf(value));
            }
        });

        // 设置请求真实IP
        String host = ObjectUtils.ifNull(session, Session::getHost);
        if (StringUtils.nonEmpty(host)) {
            template.header("X-Real-IP", host);
        }

        // 设置获取原始相应对象请求头
        FeignUtils.initializeResponseWrappingHeader(template, false);
    }
}
