package com.chelvc.framework.feign.util;

import java.util.Collection;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import feign.RequestTemplate;
import feign.Response;
import lombok.NonNull;

/**
 * Feign工具类
 *
 * @author Woody
 * @date 2023/4/5
 */
public final class FeignUtils {
    private FeignUtils() {
    }

    /**
     * 判断是否包含返回原始结果请求头
     *
     * @param response 请求响应对象
     * @return true/false
     */
    public static boolean isResponseWrappingHeader(@NonNull Response response) {
        Collection<String> values = ObjectUtils.ifNull(
                response.headers(), headers -> headers.get(SessionContextHolder.HEADER_RESPONSE_WRAPPING)
        );
        return values != null && values.stream().anyMatch(StringUtils.TRUE::equals);
    }

    /**
     * 初始化结果包装请求头
     *
     * @param template http请求模版对象
     */
    public static void initializeResponseWrappingHeader(@NonNull RequestTemplate template) {
        initializeResponseWrappingHeader(template, true);
    }

    /**
     * 初始化结果包装请求头
     *
     * @param template http请求模版对象
     * @param wrapping 是否包装请求结果
     */
    public static void initializeResponseWrappingHeader(@NonNull RequestTemplate template, boolean wrapping) {
        template.header(SessionContextHolder.HEADER_RESPONSE_WRAPPING, wrapping ? StringUtils.TRUE : StringUtils.FALSE);
    }
}
