package com.chelvc.framework.oauth.token;

import java.time.Duration;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.oauth.context.OauthContextHolder;
import com.chelvc.framework.redis.util.RedisUtils;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.OAuth2RefreshToken;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.security.oauth2.provider.token.store.JwtAccessTokenConverter;
import org.springframework.security.oauth2.provider.token.store.JwtTokenStore;

/**
 * 基于Redis的令牌存储实现
 *
 * @author Woody
 * @date 2023/4/5
 */
@Slf4j
public class RedisTokenStore extends JwtTokenStore {
    private final RedisTemplate<String, Object> redisTemplate;

    public RedisTokenStore(@NonNull RedisTemplate<String, Object> redisTemplate,
                           @NonNull JwtAccessTokenConverter jwtAccessTokenConverter) {
        super(jwtAccessTokenConverter);
        this.redisTemplate = redisTemplate;
    }

    @Override
    public OAuth2RefreshToken readRefreshToken(String tokenValue) {
        OAuth2RefreshToken refreshToken = super.readRefreshToken(tokenValue);
        OauthContextHolder.setRefreshToken(refreshToken);
        return refreshToken;
    }

    @Override
    public void storeAccessToken(OAuth2AccessToken token, OAuth2Authentication authentication) {
        String key = OauthContextHolder.buildAccessTokenKey(
                authentication.getPrincipal(), SessionContextHolder.getTerminal()
        );
        if (log.isDebugEnabled()) {
            log.debug("Saving redis token: {}", key);
        }
        Duration duration = RedisUtils.duration(token.getExpiration());
        this.redisTemplate.opsForValue().set(key, token.getValue(), duration);
    }
}
